@@ -1716,7 +1716,8 @@ static void I_InitGraphicsMode(void)
     SDL_RendererInfo info;
     if (SDL_GetRendererInfo(renderer, &info) == 0)
     {
-        I_Printf(VB_DEBUG, "SDL render driver: %s", info.name);
+        I_Printf(VB_DEBUG, "SDL render driver: %s (%s)", info.name,
+                 SDL_GetCurrentVideoDriver());
 #ifdef _WIN32
         d3d_renderer = !strncmp(info.name, "direct3d", strlen(info.name));
 #endif
