@@ -30,8 +30,10 @@
 #include "doomdef.h"
 #include "doomstat.h"
 #include "g_game.h"
+#include "g_umapinfo.h"
 #include "info.h"
 #include "m_cheat.h"
+#include "m_array.h"
 #include "m_fixed.h"
 #include "m_input.h"
 #include "m_misc.h"
@@ -50,7 +52,6 @@
 #include "sounds.h"
 #include "st_widgets.h"
 #include "tables.h"
-#include "u_mapinfo.h"
 #include "w_wad.h"
 #include "ws_stuff.h"
 
@@ -63,53 +64,53 @@
 //-----------------------------------------------------------------------------
 
 static void cheat_mus(char *buf);
-static void cheat_choppers();
-static void cheat_god();
-static void cheat_fa();
-static void cheat_k();
-static void cheat_kfa();
-static void cheat_noclip();
+static void cheat_choppers(void);
+static void cheat_god(void);
+static void cheat_fa(void);
+static void cheat_k(void);
+static void cheat_kfa(void);
+static void cheat_noclip(void);
 static void cheat_pw(int pw);
-static void cheat_behold();
+static void cheat_behold(void);
 static void cheat_clev(char *buf);
-static void cheat_clev0();
-static void cheat_mypos();
+static void cheat_clev0(void);
+static void cheat_mypos(void);
 static void cheat_comp(char *buf);
-static void cheat_comp0();
+static void cheat_comp0(void);
 static void cheat_skill(char *buf);
-static void cheat_skill0();
-static void cheat_friction();
-static void cheat_pushers();
-static void cheat_tran();
-static void cheat_massacre();
-static void cheat_ddt();
-static void cheat_hom();
-static void cheat_fast();
-static void cheat_key();
-static void cheat_keyx();
+static void cheat_skill0(void);
+static void cheat_friction(void);
+static void cheat_pushers(void);
+static void cheat_tran(void);
+static void cheat_massacre(void);
+static void cheat_ddt(void);
+static void cheat_hom(void);
+static void cheat_fast(void);
+static void cheat_key(void);
+static void cheat_keyx(void);
 static void cheat_keyxx(int key);
-static void cheat_weap();
+static void cheat_weap(void);
 static void cheat_weapx(char *buf);
-static void cheat_ammo();
+static void cheat_ammo(void);
 static void cheat_ammox(char *buf);
-static void cheat_smart();
-static void cheat_pitch();
-static void cheat_nuke();
-static void cheat_rate();
-static void cheat_buddha();
-static void cheat_spechits();
-static void cheat_notarget();
-static void cheat_freeze();
-static void cheat_health();
-static void cheat_megaarmour();
-static void cheat_reveal_secret();
-static void cheat_reveal_kill();
-static void cheat_reveal_item();
+static void cheat_smart(void);
+static void cheat_pitch(void);
+static void cheat_nuke(void);
+static void cheat_rate(void);
+static void cheat_buddha(void);
+static void cheat_spechits(void);
+static void cheat_notarget(void);
+static void cheat_freeze(void);
+static void cheat_health(void);
+static void cheat_megaarmour(void);
+static void cheat_reveal_secret(void);
+static void cheat_reveal_kill(void);
+static void cheat_reveal_item(void);
 
-static void cheat_autoaim();      // killough 7/19/98
-static void cheat_tst();
-static void cheat_showfps(); // [FG] FPS counter widget
-static void cheat_speed();
+static void cheat_autoaim(void);      // killough 7/19/98
+static void cheat_tst(void);
+static void cheat_showfps(void); // [FG] FPS counter widget
+static void cheat_speed(void);
 
 //-----------------------------------------------------------------------------
 //
@@ -134,201 +135,201 @@ static void cheat_speed();
 
 struct cheat_s cheat[] = {
   {"idmus",      "Change music",      always,
-   {cheat_mus}, -2 },
+   {.s = cheat_mus}, -2 },
 
   {"idchoppers", "Chainsaw",          not_net | not_demo,
-   {cheat_choppers} },
+   {.v = cheat_choppers} },
 
   {"iddqd",      "God mode",          not_net | not_demo,
-   {cheat_god} },
+   {.v = cheat_god} },
 
   {"buddha",     "Buddha mode",       not_net | not_demo,
-   {cheat_buddha} },
+   {.v = cheat_buddha} },
 
   {"idk",        NULL,                not_net | not_demo | not_deh,
-   {cheat_k} }, // The most controversial cheat code in Doom history!!!
+   {.v = cheat_k} }, // The most controversial cheat code in Doom history!!!
 
   {"idkfa",      "Ammo & Keys",       not_net | not_demo,
-   {cheat_kfa} },
+   {.v = cheat_kfa} },
 
   {"idfa",       "Ammo",              not_net | not_demo,
-   {cheat_fa} },
+   {.v = cheat_fa} },
 
   {"idspispopd", "No Clipping 1",     not_net | not_demo,
-   {cheat_noclip} },
+   {.v = cheat_noclip} },
 
   {"idclip",     "No Clipping 2",     not_net | not_demo,
-   {cheat_noclip} },
+   {.v = cheat_noclip} },
 
   {"idbeholdo",  NULL,                not_net | not_demo | not_deh,
-   {cheat_pw}, NUMPOWERS }, // [FG] disable all powerups at once
+   {.i = cheat_pw}, NUMPOWERS }, // [FG] disable all powerups at once
 
   {"idbeholdh",  "Health",            not_net | not_demo,
-   {cheat_health} },
+   {.v = cheat_health} },
 
   {"idbeholdm",  "Mega Armor",        not_net | not_demo,
-   {cheat_megaarmour} },
+   {.v = cheat_megaarmour} },
 
   {"idbeholdv",  "Invincibility",     not_net | not_demo,
-   {cheat_pw}, pw_invulnerability },
+   {.i = cheat_pw}, pw_invulnerability },
 
   {"idbeholds",  "Berserk",           not_net | not_demo,
-   {cheat_pw}, pw_strength },
+   {.i = cheat_pw}, pw_strength },
 
   {"idbeholdi",  "Invisibility",      not_net | not_demo,  
-   {cheat_pw}, pw_invisibility },
+   {.i = cheat_pw}, pw_invisibility },
 
   {"idbeholdr",  "Radiation Suit",    not_net | not_demo,
-   {cheat_pw}, pw_ironfeet },
+   {.i = cheat_pw}, pw_ironfeet },
 
   {"idbeholda",  "Auto-map",          not_dm,
-   {cheat_pw}, pw_allmap },
+   {.i = cheat_pw}, pw_allmap },
 
   {"idbeholdl",  "Lite-Amp Goggles",  not_dm,
-   {cheat_pw}, pw_infrared },
+   {.i = cheat_pw}, pw_infrared },
 
   {"idbehold",   "BEHOLD menu",       not_net | not_demo,
-   {cheat_behold} },
+   {.v = cheat_behold} },
 
   {"idclev",     "Level Warp",        not_net | not_demo | not_menu,
-   {cheat_clev}, -2 },
+   {.s = cheat_clev}, -2 },
 
   {"idclev",     "Level Warp",        not_net | not_demo | not_menu,
-   {cheat_clev0} },
+   {.v = cheat_clev0} },
 
   {"idmypos",    "Player Position",   not_dm, // [FG] not_net | not_demo,
-   {cheat_mypos} },
+   {.v = cheat_mypos} },
 
   {"comp",    NULL,                   not_net | not_demo | not_menu,
-   {cheat_comp}, -2 },
+   {.s = cheat_comp}, -2 },
 
   {"comp",    NULL,                   not_net | not_demo | not_menu,
-   {cheat_comp0} },
+   {.v = cheat_comp0} },
 
   {"skill",    NULL,                  not_net | not_demo | not_menu,
-   {cheat_skill}, -1 },
+   {.s = cheat_skill}, -1 },
 
   {"skill",    NULL,                  not_net | not_demo | not_menu,
-   {cheat_skill0} },
+   {.v = cheat_skill0} },
 
   {"killem",     NULL,                not_net | not_demo,
-   {cheat_massacre} },   // jff 2/01/98 kill all monsters
+   {.v = cheat_massacre} },   // jff 2/01/98 kill all monsters
 
   {"spechits",     NULL,              not_net | not_demo,
-   {cheat_spechits} },
+   {.v = cheat_spechits} },
 
   {"notarget",   "Notarget mode",     not_net | not_demo,
-   {cheat_notarget} },
+   {.v = cheat_notarget} },
 
   {"freeze",     "Freeze",            not_net | not_demo,
-   {cheat_freeze} },
+   {.v = cheat_freeze} },
 
   {"iddt",       "Map cheat",         not_dm,
-   {cheat_ddt} },        // killough 2/07/98: moved from am_map.c
+   {.v = cheat_ddt} },        // killough 2/07/98: moved from am_map.c
 
   {"iddst",      NULL,                not_dm,
-   {cheat_reveal_secret} },
+   {.v = cheat_reveal_secret} },
 
   {"iddkt",      NULL,                not_dm,
-   {cheat_reveal_kill} },
+   {.v = cheat_reveal_kill} },
 
   {"iddit",      NULL,                not_dm,
-   {cheat_reveal_item} },
+   {.v = cheat_reveal_item} },
 
   {"hom",     NULL,                   always,
-   {cheat_hom} },        // killough 2/07/98: HOM autodetector
+   {.v = cheat_hom} },        // killough 2/07/98: HOM autodetector
 
   {"key",     NULL,                   not_net | not_demo, 
-   {cheat_key} },     // killough 2/16/98: generalized key cheats
+   {.v = cheat_key} },     // killough 2/16/98: generalized key cheats
 
   {"keyr",    NULL,                   not_net | not_demo,
-   {cheat_keyx} },
+   {.v = cheat_keyx} },
 
   {"keyy",    NULL,                   not_net | not_demo,
-   {cheat_keyx} },
+   {.v = cheat_keyx} },
 
   {"keyb",    NULL,                   not_net | not_demo,
-   {cheat_keyx} },
+   {.v = cheat_keyx} },
 
   {"keyrc",   NULL,                   not_net | not_demo, 
-   {cheat_keyxx}, it_redcard },
+   {.i = cheat_keyxx}, it_redcard },
 
   {"keyyc",   NULL,                   not_net | not_demo,
-   {cheat_keyxx}, it_yellowcard },
+   {.i = cheat_keyxx}, it_yellowcard },
 
   {"keybc",   NULL,                   not_net | not_demo, 
-   {cheat_keyxx}, it_bluecard },
+   {.i = cheat_keyxx}, it_bluecard },
 
   {"keyrs",   NULL,                   not_net | not_demo,
-   {cheat_keyxx}, it_redskull },
+   {.i = cheat_keyxx}, it_redskull },
 
   {"keyys",   NULL,                   not_net | not_demo,
-   {cheat_keyxx}, it_yellowskull },
+   {.i = cheat_keyxx}, it_yellowskull },
 
   {"keybs",   NULL,                   not_net | not_demo,
-   {cheat_keyxx}, it_blueskull }, // killough 2/16/98: end generalized keys
+   {.i = cheat_keyxx}, it_blueskull }, // killough 2/16/98: end generalized keys
 
   {"weap",    NULL,                   not_net | not_demo,
-   {cheat_weap} },    // killough 2/16/98: generalized weapon cheats
+   {.v = cheat_weap} },    // killough 2/16/98: generalized weapon cheats
 
   {"weap",    NULL,                   not_net | not_demo,
-   {cheat_weapx}, -1 },
+   {.s = cheat_weapx}, -1 },
 
   {"ammo",    NULL,                   not_net | not_demo,
-   {cheat_ammo} },
+   {.v = cheat_ammo} },
 
   {"ammo",    NULL,                   not_net | not_demo,
-   {cheat_ammox}, -1 }, // killough 2/16/98: end generalized weapons
+   {.s = cheat_ammox}, -1 }, // killough 2/16/98: end generalized weapons
 
   {"tran",    NULL,                   always,
-   {cheat_tran} },    // invoke translucency         // phares
+   {.v = cheat_tran} },    // invoke translucency         // phares
 
   {"smart",   NULL,                   not_net | not_demo,
-   {cheat_smart} },      // killough 2/21/98: smart monster toggle
+   {.v = cheat_smart} },      // killough 2/21/98: smart monster toggle
 
   {"pitch",   NULL,                   always,
-   {cheat_pitch} },      // killough 2/21/98: pitched sound toggle
+   {.v = cheat_pitch} },      // killough 2/21/98: pitched sound toggle
 
   // killough 2/21/98: reduce RSI injury by adding simpler alias sequences:
   {"mbfran",     NULL,                always, 
-   {cheat_tran} },    // killough 2/21/98: same as mbftran
+   {.v = cheat_tran} },    // killough 2/21/98: same as mbftran
 
   {"fast",    NULL,                   not_net | not_demo,
-   {cheat_fast} },       // killough 3/6/98: -fast toggle
+   {.v = cheat_fast} },       // killough 3/6/98: -fast toggle
 
   {"ice",     NULL,                   not_net | not_demo,
-   {cheat_friction} },   // phares 3/10/98: toggle variable friction effects
+   {.v = cheat_friction} },   // phares 3/10/98: toggle variable friction effects
 
   {"push",    NULL,                   not_net | not_demo, 
-   {cheat_pushers} },    // phares 3/10/98: toggle pushers
+   {.v = cheat_pushers} },    // phares 3/10/98: toggle pushers
 
   {"nuke",    NULL,                   not_net | not_demo,
-   {cheat_nuke} },       // killough 12/98: disable nukage damage
+   {.v = cheat_nuke} },       // killough 12/98: disable nukage damage
 
   {"rate",    NULL,                   always,
-   {cheat_rate} },
+   {.v = cheat_rate} },
 
   {"aim",        NULL,                not_net | not_demo | beta_only,
-   {cheat_autoaim} },
+   {.v = cheat_autoaim} },
 
   {"eek",        NULL,                not_dm  | not_demo | beta_only,
-   {cheat_ddt} },        // killough 2/07/98: moved from am_map.c
+   {.v = cheat_ddt} },        // killough 2/07/98: moved from am_map.c
 
   {"amo",        NULL,                not_net | not_demo | beta_only,
-   {cheat_kfa} },
+   {.v = cheat_kfa} },
 
   {"tst",        NULL,                not_net | not_demo | beta_only,
-   {cheat_tst} },
+   {.v = cheat_tst} },
 
   {"nc",         NULL,                not_net | not_demo | beta_only,
-   {cheat_noclip} },
+   {.v = cheat_noclip} },
 
 // [FG] FPS counter widget
   {"showfps",    NULL,                always,
-   {cheat_showfps} },
+   {.v = cheat_showfps} },
 
   {"speed",      NULL,                not_dm,
-   {cheat_speed} },
+   {.v = cheat_speed} },
 
   {NULL}                 // end-of-list marker
 };
@@ -336,18 +337,18 @@ struct cheat_s cheat[] = {
 //-----------------------------------------------------------------------------
 
 // [FG] FPS counter widget
-static void cheat_showfps()
+static void cheat_showfps(void)
 {
   plyr->cheats ^= CF_SHOWFPS;
 }
 
-static void cheat_speed()
+static void cheat_speed(void)
 {
   speedometer = (speedometer + 1) % 4;
 }
 
 // killough 7/19/98: Autoaiming optional in beta emulation mode
-static void cheat_autoaim()
+static void cheat_autoaim(void)
 {
   displaymsg((autoaim=!autoaim) ?
     "Projectile autoaiming on" : 
@@ -416,14 +417,14 @@ static void cheat_mus(char *buf)
 }
 
 // 'choppers' invulnerability & chainsaw
-static void cheat_choppers()
+static void cheat_choppers(void)
 {
   plyr->weaponowned[wp_chainsaw] = true;
   plyr->powers[pw_invulnerability] = true;
   displaymsg("%s", s_STSTR_CHOPPERS); // Ty 03/27/98 - externalized
 }
 
-static void cheat_god()
+static void cheat_god(void)
 {                                    // 'dqd' cheat for toggleable god mode
   // [crispy] dead players are first respawned at the current position
   if (plyr->playerstate == PST_DEAD)
@@ -463,7 +464,7 @@ static void cheat_god()
     displaymsg("%s", s_STSTR_DQDOFF); // Ty 03/27/98 - externalized
 }
 
-static void cheat_buddha()
+static void cheat_buddha(void)
 {
   plyr->cheats ^= CF_BUDDHA;
   if (plyr->cheats & CF_BUDDHA)
@@ -472,7 +473,7 @@ static void cheat_buddha()
     displaymsg("Buddha Mode OFF");
 }
 
-static void cheat_notarget()
+static void cheat_notarget(void)
 {
   plyr->cheats ^= CF_NOTARGET;
   if (plyr->cheats & CF_NOTARGET)
@@ -483,7 +484,7 @@ static void cheat_notarget()
 
 boolean frozen_mode;
 
-static void cheat_freeze()
+static void cheat_freeze(void)
 {
   frozen_mode = !frozen_mode;
   if (frozen_mode)
@@ -492,7 +493,7 @@ static void cheat_freeze()
     displaymsg("Freeze OFF");
 }
 
-static void cheat_avj()
+static void cheat_avj(void)
 {
   void A_VileJump(mobj_t *mo);
 
@@ -501,7 +502,7 @@ static void cheat_avj()
 }
 
 // CPhipps - new health and armour cheat codes
-static void cheat_health()
+static void cheat_health(void)
 {
   if (!(plyr->cheats & CF_GODMODE))
   {
@@ -512,20 +513,20 @@ static void cheat_health()
   }
 }
 
-static void cheat_megaarmour()
+static void cheat_megaarmour(void)
 {
   plyr->armorpoints = idfa_armor;      // Ty 03/09/98 - deh
   plyr->armortype = idfa_armor_class;  // Ty 03/09/98 - deh
   displaymsg("%s", s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
 }
 
-static void cheat_tst()
+static void cheat_tst(void)
 { // killough 10/98: same as iddqd except for message
   cheat_god();
   displaymsg(plyr->cheats & CF_GODMODE ? "God Mode On" : "God Mode Off");
 }
 
-static void cheat_fa()
+static void cheat_fa(void)
 {
   int i;
 
@@ -552,7 +553,7 @@ static void cheat_fa()
   displaymsg("%s", s_STSTR_FAADDED);
 }
 
-static void cheat_k()
+static void cheat_k(void)
 {
   int i;
   for (i=0;i<NUMCARDS;i++)
@@ -563,14 +564,14 @@ static void cheat_k()
       }
 }
 
-static void cheat_kfa()
+static void cheat_kfa(void)
 {
   cheat_k();
   cheat_fa();
   displaymsg("%s", s_STSTR_KFAADDED);
 }
 
-static void cheat_noclip()
+static void cheat_noclip(void)
 {
   // Simplified, accepting both "noclip" and "idspispopd".
   // no clipping mode cheat
@@ -600,13 +601,13 @@ static void cheat_pw(int pw)
 }
 
 // 'behold' power-up menu
-static void cheat_behold()
+static void cheat_behold(void)
 {
   displaymsg("%s", s_STSTR_BEHOLD); // Ty 03/27/98 - externalized
 }
 
 // 'clev' change-level cheat
-static void cheat_clev0()
+static void cheat_clev0(void)
 {
   int epsd, map;
   char *cur, *next;
@@ -686,14 +687,14 @@ static void cheat_clev(char *buf)
 
 // 'mypos' for player position
 // killough 2/7/98: simplified using dprintf and made output more user-friendly
-static void cheat_mypos()
+static void cheat_mypos(void)
 {
   plyr->cheats ^= CF_MAPCOORDS;
   if ((plyr->cheats & CF_MAPCOORDS) == 0)
     plyr->message = "";
 }
 
-void cheat_mypos_print()
+void cheat_mypos_print(void)
 {
   displaymsg("X=%.10f Y=%.10f A=%-.0f",
           (double)players[consoleplayer].mo->x / FRACUNIT,
@@ -703,7 +704,7 @@ void cheat_mypos_print()
 
 // compatibility cheat
 
-static void cheat_comp0()
+static void cheat_comp0(void)
 {
   displaymsg("Complevel: %s", G_GetCurrentComplevelName());
 }
@@ -728,14 +729,14 @@ static void cheat_comp(char *buf)
 }
 
 // variable friction cheat
-static void cheat_friction()
+static void cheat_friction(void)
 {
   displaymsg(            // Ty 03/27/98 - *not* externalized
     (variable_friction = !variable_friction) ? "Variable Friction enabled" : 
                                                "Variable Friction disabled");
 }
 
-static void cheat_skill0()
+static void cheat_skill0(void)
 {
   displaymsg("Skill: %s", default_skill_strings[gameskill + 1]);
 }
@@ -756,20 +757,20 @@ static void cheat_skill(char *buf)
 
 // Pusher cheat
 // phares 3/10/98
-static void cheat_pushers()
+static void cheat_pushers(void)
 {
   displaymsg(           // Ty 03/27/98 - *not* externalized
     (allow_pushers = !allow_pushers) ? "Pushers enabled" : "Pushers disabled");
 }
 
 // translucency cheat
-static void cheat_tran()
+static void cheat_tran(void)
 {
   displaymsg(            // Ty 03/27/98 - *not* externalized
     (translucency = !translucency) ? "Translucency enabled" : "Translucency disabled");
 }
 
-static void cheat_massacre()    // jff 2/01/98 kill all monsters
+static void cheat_massacre(void)    // jff 2/01/98 kill all monsters
 {
   // jff 02/01/98 'em' cheat - kill all monsters
   // partially taken from Chi's .46 port
@@ -807,7 +808,7 @@ static void cheat_massacre()    // jff 2/01/98 kill all monsters
   displaymsg("%d Monster%s Killed", killcount, killcount==1 ? "" : "s");
 }
 
-static void cheat_spechits()
+static void cheat_spechits(void)
 {
   int i, speciallines = 0;
   boolean origcards[NUMCARDS];
@@ -876,7 +877,7 @@ static void cheat_spechits()
     plyr->cards[i] = origcards[i];
   }
 
-  if (gamemapinfo && gamemapinfo->numbossactions > 0)
+  if (gamemapinfo && array_size(gamemapinfo->bossactions))
   {
     thinker_t *th;
 
@@ -886,13 +887,14 @@ static void cheat_spechits()
       {
         mobj_t *mo = (mobj_t *) th;
 
-        for (i = 0; i < gamemapinfo->numbossactions; i++)
+        bossaction_t *bossaction;
+        array_foreach(bossaction, gamemapinfo->bossactions)
         {
-          if (gamemapinfo->bossactions[i].type == mo->type)
+          if (bossaction->type == mo->type)
           {
             dummy = *lines;
-            dummy.special = (short)gamemapinfo->bossactions[i].special;
-            dummy.tag = (short)gamemapinfo->bossactions[i].tag;
+            dummy.special = (short)bossaction->special;
+            dummy.tag = (short)bossaction->tag;
             // use special semantics for line activation to block problem types.
             if (!P_UseSpecialLine(mo, &dummy, 0, true))
               P_CrossSpecialLine(&dummy, 0, mo, true);
@@ -966,13 +968,13 @@ static void cheat_spechits()
 
 // killough 2/7/98: move iddt cheat from am_map.c to here
 // killough 3/26/98: emulate Doom better
-static void cheat_ddt()
+static void cheat_ddt(void)
 {
   if (automapactive)
     ddt_cheating = (ddt_cheating+1) % 3;
 }
 
-static void cheat_reveal_secret()
+static void cheat_reveal_secret(void)
 {
   static int last_secret = -1;
 
@@ -1048,7 +1050,7 @@ static void cheat_cycle_mobj(mobj_t **last_mobj, int *last_count,
   } while (th != start_th);
 }
 
-static void cheat_reveal_kill()
+static void cheat_reveal_kill(void)
 {
   if (automapactive)
   {
@@ -1059,7 +1061,7 @@ static void cheat_reveal_kill()
   }
 }
 
-static void cheat_reveal_item()
+static void cheat_reveal_item(void)
 {
   if (automapactive)
   {
@@ -1071,14 +1073,14 @@ static void cheat_reveal_item()
 }
 
 // killough 2/7/98: HOM autodetection
-static void cheat_hom()
+static void cheat_hom(void)
 {
   displaymsg((autodetect_hom = !autodetect_hom) ? "HOM Detection On" :
     "HOM Detection Off");
 }
 
 // killough 3/6/98: -fast parameter toggle
-static void cheat_fast()
+static void cheat_fast(void)
 {
   displaymsg((fastparm = !fastparm) ? "Fast Monsters On" : 
     "Fast Monsters Off");  // Ty 03/27/98 - *not* externalized
@@ -1086,12 +1088,12 @@ static void cheat_fast()
 }
 
 // killough 2/16/98: keycard/skullkey cheat functions
-static void cheat_key()
+static void cheat_key(void)
 {
   displaymsg("Red, Yellow, Blue");  // Ty 03/27/98 - *not* externalized
 }
 
-static void cheat_keyx()
+static void cheat_keyx(void)
 {
   displaymsg("Card, Skull");        // Ty 03/27/98 - *not* externalized
 }
@@ -1104,7 +1106,7 @@ static void cheat_keyxx(int key)
 
 // killough 2/16/98: generalized weapon cheats
 
-static void cheat_weap()
+static void cheat_weap(void)
 {                                   // Ty 03/27/98 - *not* externalized
   displaymsg(ALLOW_SSG ? // killough 2/28/98
     "Weapon number 1-9" : "Weapon number 1-8");
@@ -1135,7 +1137,7 @@ static void cheat_weapx(char *buf)
 }
 
 // killough 2/16/98: generalized ammo cheats
-static void cheat_ammo()
+static void cheat_ammo(void)
 {
   displaymsg("Ammo 1-4, Backpack");  // Ty 03/27/98 - *not* externalized
 }
@@ -1167,25 +1169,25 @@ static void cheat_ammox(char *buf)
       }
 }
 
-static void cheat_smart()
+static void cheat_smart(void)
 {
   displaymsg((monsters_remember = !monsters_remember) ? 
     "Smart Monsters Enabled" : "Smart Monsters Disabled");
 }
 
-static void cheat_pitch()
+static void cheat_pitch(void)
 {
   displaymsg((pitched_sounds = !pitched_sounds) ? "Pitch Effects Enabled" :
     "Pitch Effects Disabled");
 }
 
-static void cheat_nuke()
+static void cheat_nuke(void)
 {
   displaymsg((disable_nuke = !disable_nuke) ? "Nukage Disabled" :
     "Nukage Enabled");
 }
 
-static void cheat_rate()
+static void cheat_rate(void)
 {
   plyr->cheats ^= CF_RENDERSTATS;
 }
@@ -1286,21 +1288,21 @@ static const struct {
   const cheatf_t func;
   const int arg;
 } cheat_input[] = {
-  { input_iddqd,     not_net|not_demo, {cheat_god},      0 },
-  { input_idkfa,     not_net|not_demo, {cheat_kfa},      0 },
-  { input_idfa,      not_net|not_demo, {cheat_fa},       0 },
-  { input_idclip,    not_net|not_demo, {cheat_noclip},   0 },
-  { input_idbeholdh, not_net|not_demo, {cheat_health},   0 },
-  { input_idbeholdm, not_net|not_demo, {cheat_megaarmour}, 0 },
-  { input_idbeholdv, not_net|not_demo, {cheat_pw},       pw_invulnerability },
-  { input_idbeholds, not_net|not_demo, {cheat_pw},       pw_strength },
-  { input_idbeholdi, not_net|not_demo, {cheat_pw},       pw_invisibility },
-  { input_idbeholdr, not_net|not_demo, {cheat_pw},       pw_ironfeet },
-  { input_idbeholdl, not_dm,           {cheat_pw},       pw_infrared },
-  { input_iddt,      not_dm,           {cheat_ddt},      0 },
-  { input_notarget,  not_net|not_demo, {cheat_notarget}, 0 },
-  { input_freeze,    not_net|not_demo, {cheat_freeze},   0 },
-  { input_avj,       not_net|not_demo, {cheat_avj},      0 },
+  { input_iddqd,     not_net|not_demo, {.v = cheat_god},      0 },
+  { input_idkfa,     not_net|not_demo, {.v = cheat_kfa},      0 },
+  { input_idfa,      not_net|not_demo, {.v = cheat_fa},       0 },
+  { input_idclip,    not_net|not_demo, {.v = cheat_noclip},   0 },
+  { input_idbeholdh, not_net|not_demo, {.v = cheat_health},   0 },
+  { input_idbeholdm, not_net|not_demo, {.v = cheat_megaarmour}, 0 },
+  { input_idbeholdv, not_net|not_demo, {.i = cheat_pw},       pw_invulnerability },
+  { input_idbeholds, not_net|not_demo, {.i = cheat_pw},       pw_strength },
+  { input_idbeholdi, not_net|not_demo, {.i = cheat_pw},       pw_invisibility },
+  { input_idbeholdr, not_net|not_demo, {.i = cheat_pw},       pw_ironfeet },
+  { input_idbeholdl, not_dm,           {.i = cheat_pw},       pw_infrared },
+  { input_iddt,      not_dm,           {.v = cheat_ddt},      0 },
+  { input_notarget,  not_net|not_demo, {.v = cheat_notarget}, 0 },
+  { input_freeze,    not_net|not_demo, {.v = cheat_freeze},   0 },
+  { input_avj,       not_net|not_demo, {.v = cheat_avj},      0 },
 };
 
 boolean M_CheatResponder(event_t *ev)
