@@ -25,7 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "d_main.h"
+#include "d_iwad.h"
 #include "d_think.h"
 #include "doomdef.h"
 #include "doomstat.h"
@@ -1055,9 +1055,7 @@ void R_InitTranMap(int progress)
   int p = M_CheckParmWithArgs("-dumptranmap", 1);
   if (p > 0)
   {
-      char *path = malloc(strlen(myargv[p + 1]) + 5);
-      strcpy(path, myargv[p + 1]);
-      AddDefaultExtension(path, ".lmp");
+      char *path = AddDefaultExtension(myargv[p + 1], ".lmp");
 
       M_WriteFile(path, main_tranmap, 256 * 256);
 
