@@ -619,7 +619,7 @@ static void UpdateFace(sbe_face_t *face, player_t *player)
             boolean right = false;
 
             // [FG] show "Ouch Face" as intended
-            if (player->health - face->oldhealth > ST_MUCHPAIN)
+            if (face->oldhealth - player->health > ST_MUCHPAIN)
             {
                 // [FG] raise "Ouch Face" priority
                 priority = 8;
@@ -672,7 +672,7 @@ static void UpdateFace(sbe_face_t *face, player_t *player)
         // getting hurt because of your own damn stupidity
         if (player->damagecount)
         {
-            if (player->health - face->oldhealth > ST_MUCHPAIN)
+            if (face->oldhealth - player->health > ST_MUCHPAIN)
             {
                 priority = 7;
                 face->facecount = ST_TURNCOUNT;
@@ -1020,6 +1020,11 @@ static void UpdateStatusBar(player_t *player)
 
     int barindex = MAX(screenblocks - 10, 0);
 
+    if (barindex >= array_size(sbardef->statusbars))
+    {
+        barindex = array_size(sbardef->statusbars) - 1;
+    }
+
     if (automapactive && automapoverlay == AM_OVERLAY_OFF)
     {
         barindex = 0;
@@ -1781,11 +1786,6 @@ void ST_Drawer(void)
     }
 
     DrawStatusBar();
-
-    if (hud_crosshair)
-    {
-        HU_DrawCrosshair();
-    }
 }
 
 void ST_Start(void)
@@ -1833,6 +1833,35 @@ void ST_InitRes(void)
                  PU_RENDERER, 0);
 }
 
+const char **ST_StatusbarList(void)
+{
+    if (!sbardef)
+    {
+        return NULL;
+    }
+
+    static const char **strings;
+
+    if (array_size(strings))
+    {
+        return strings;
+    }
+
+    statusbar_t *item;
+    array_foreach(item, sbardef->statusbars)
+    {
+        if (item->fullscreenrender)
+        {
+            array_push(strings, "Fullscreen");
+        }
+        else
+        {
+            array_push(strings, "Status Bar");
+        }
+    }
+    return strings;
+}
+
 void ST_ResetPalette(void)
 {
     I_SetPalette(W_CacheLumpName("PLAYPAL", PU_CACHE));
@@ -1872,7 +1901,7 @@ void ST_BindSTSVariables(void)
   M_BindBool("sts_colored_numbers", &sts_colored_numbers, NULL,
              false, ss_stat, wad_yes, "Colored numbers on the status bar");
   M_BindBool("sts_pct_always_gray", &sts_pct_always_gray, NULL,
-             false, ss_stat, wad_yes,
+             false, ss_none, wad_yes,
              "Percent signs on the status bar are always gray");
   M_BindBool("st_solidbackground", &st_solidbackground, NULL,
              false, ss_stat, wad_no,
