@@ -24,6 +24,7 @@
 #include "doomstat.h"
 #include "doomtype.h"
 #include "dstrings.h"
+#include "g_umapinfo.h"
 #include "hu_command.h"
 #include "hu_coordinates.h"
 #include "hu_obituary.h"
@@ -41,7 +42,6 @@
 #include "sounds.h"
 #include "st_sbardef.h"
 #include "st_stuff.h"
-#include "u_mapinfo.h"
 #include "v_video.h"
 
 boolean       show_messages;
@@ -594,10 +594,11 @@ void ST_ResetTitle(void)
             s = gamemapinfo->mapname;
         }
 
-        if (s == gamemapinfo->mapname || U_CheckField(s))
+        if (!(gamemapinfo->flags & MapInfo_LabelClear))
         {
             M_snprintf(string, sizeof(string), "%s: ", s);
         }
+
         s = gamemapinfo->levelname;
     }
     else if (gamestate == GS_LEVEL)
@@ -646,7 +647,7 @@ void ST_ResetTitle(void)
     announce_string[0] = '\0';
     if (hud_map_announce && leveltime == 0)
     {
-        if (gamemapinfo && U_CheckField(gamemapinfo->author))
+        if (gamemapinfo && gamemapinfo->author)
         {
             M_snprintf(announce_string, sizeof(announce_string), "%s by %s",
                        string, gamemapinfo->author);
